@@ -23,6 +23,7 @@
 #include <string>
 #include <string.h>
 #include <stdint.h>
+#include <stdlib.h>
 
 using std::string;
 
@@ -32,7 +33,32 @@ namespace Crypto {
   class CryptoException {
   public:
     string text;
-    CryptoException( string s_text ) : text( s_text ) {};
+    bool fatal;
+    CryptoException( string s_text, bool s_fatal = false )
+      : text( s_text ), fatal( s_fatal ) {};
+  };
+
+  /* 16-byte-aligned buffer, with length. */
+  class AlignedBuffer {
+  private:
+    size_t m_len;
+    void *m_allocated;
+    char *m_data;
+
+  public:
+    AlignedBuffer( size_t len, const char *data = NULL );
+
+    ~AlignedBuffer() {
+      free( m_allocated );
+    }
+
+    char * data( void ) const { return m_data; }
+    size_t len( void )  const { return m_len;  }
+
+  private:
+    /* Not implemented */
+    AlignedBuffer( const AlignedBuffer& );
+    AlignedBuffer& operator=( const AlignedBuffer& );
   };
 
   class Base64Key {
@@ -47,8 +73,11 @@ namespace Crypto {
   };
 
   class Nonce {
+  public:
+    static const int NONCE_LEN = 12;
+
   private:
-    char bytes[ 12 ] __attribute__((__aligned__ (16)));
+    char bytes[ NONCE_LEN ];
 
   public:
     Nonce( uint64_t val );
@@ -72,9 +101,17 @@ namespace Crypto {
   class Session {
   private:
     Base64Key key;
+    AlignedBuffer ctx_buf;
     ae_ctx *ctx;
+    uint64_t blocks_encrypted;
+
+    AlignedBuffer plaintext_buffer;
+    AlignedBuffer ciphertext_buffer;
+    AlignedBuffer nonce_buffer;
     
   public:
+    static const int RECEIVE_MTU = 2048;
+
     Session( Base64Key s_key );
     ~Session();
     
@@ -86,6 +123,7 @@ namespace Crypto {
   };
 
   void disable_dumping_core( void );
+  void reenable_dumping_core( void );
 }
 
 #endif
