@@ -24,7 +24,6 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/poll.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <pwd.h>
@@ -37,13 +36,13 @@
 #include <util.h>
 #endif
 
-#include "sigfd.h"
 #include "stmclient.h"
 #include "swrite.h"
 #include "completeterminal.h"
 #include "user.h"
 #include "fatal_assert.h"
 #include "locale_utils.h"
+#include "select.h"
 
 #include "networktransport.cc"
 
@@ -54,7 +53,7 @@ void STMClient::init( void )
     string native_charset( locale_charset() );
 
     fprintf( stderr, "mosh-client needs a UTF-8 native locale to run.\n\n" );
-    fprintf( stderr, "Unfortunately, the client's environemnt (%s) specifies\nthe character set \"%s\".\n\n", native_ctype.str().c_str(), native_charset.c_str() );
+    fprintf( stderr, "Unfortunately, the client's environment (%s) specifies\nthe character set \"%s\".\n\n", native_ctype.str().c_str(), native_charset.c_str() );
     int unused __attribute((unused)) = system( "locale" );
     exit( 1 );
   }
@@ -113,7 +112,7 @@ void STMClient::shutdown( void )
   }
 
   if ( still_connecting() ) {
-    fprintf( stderr, "mosh did not make a successful connection to %s:%d.\n", ip.c_str(), port );
+    fprintf( stderr, "\nmosh did not make a successful connection to %s:%d.\n", ip.c_str(), port );
     fprintf( stderr, "Please verify that UDP port %d is not firewalled and can reach the server.\n\n", port );
     fprintf( stderr, "(By default, mosh uses a UDP port between 60000 and 61000. The -p option\nselects a specific UDP port number.)\n" );
   } else if ( network ) {
@@ -125,20 +124,14 @@ void STMClient::shutdown( void )
 
 void STMClient::main_init( void )
 {
-  /* establish a fd for signals */
-  signal_fd = sigfd_init();
-  if ( signal_fd < 0 ) {
-    perror( "sigfd_init" );
-    return;
-  }
-
-  fatal_assert( sigfd_trap( SIGWINCH ) == 0 );
-  fatal_assert( sigfd_trap( SIGTERM ) == 0 );
-  fatal_assert( sigfd_trap( SIGINT ) == 0 );
-  fatal_assert( sigfd_trap( SIGHUP ) == 0 );
-  fatal_assert( sigfd_trap( SIGPIPE ) == 0 );
-  fatal_assert( sigfd_trap( SIGTSTP ) == 0 );
-  fatal_assert( sigfd_trap( SIGCONT ) == 0 );
+  Select &sel = Select::get_instance();
+  sel.add_signal( SIGWINCH );
+  sel.add_signal( SIGTERM );
+  sel.add_signal( SIGINT );
+  sel.add_signal( SIGHUP );
+  sel.add_signal( SIGPIPE );
+  sel.add_signal( SIGTSTP );
+  sel.add_signal( SIGCONT );
 
   /* get initial window size */
   if ( ioctl( STDIN_FILENO, TIOCGWINSZ, &window_size ) < 0 ) {
@@ -201,6 +194,7 @@ bool STMClient::process_network_input( void )
   
   /* Now give hints to the overlays */
   overlays.get_notification_engine().server_heard( network->get_latest_remote_state().timestamp );
+  overlays.get_notification_engine().server_acked( network->get_sent_state_acked_timestamp() );
 
   overlays.get_prediction_engine().set_local_frame_acked( network->get_sent_state_acked() );
   overlays.get_prediction_engine().set_send_interval( network->send_interval() );
@@ -298,16 +292,9 @@ void STMClient::main( void )
   main_init();
 
   /* prepare to poll for events */
-  struct pollfd pollfds[ 3 ];
-
-  pollfds[ 0 ].fd = network->fd();
-  pollfds[ 0 ].events = POLLIN;
-
-  pollfds[ 1 ].fd = STDIN_FILENO;
-  pollfds[ 1 ].events = POLLIN;
-
-  pollfds[ 2 ].fd = signal_fd;
-  pollfds[ 2 ].events = POLLIN;
+  Select &sel = Select::get_instance();
+  sel.add_fd( network->fd() );
+  sel.add_fd( STDIN_FILENO );
 
   while ( 1 ) {
     try {
@@ -320,22 +307,20 @@ void STMClient::main( void )
 	wait_time = min( 250, wait_time );
       }
 
-      int active_fds = poll( pollfds, 3, wait_time );
-      if ( active_fds < 0 && errno == EINTR ) {
-	continue;
-      } else if ( active_fds < 0 ) {
-	perror( "poll" );
+      int active_fds = sel.select( wait_time );
+      if ( active_fds < 0 ) {
+	perror( "select" );
 	break;
       }
 
-      if ( pollfds[ 0 ].revents & POLLIN ) {
+      if ( sel.read( network->fd() ) ) {
 	/* packet received from the network */
 	if ( !process_network_input() ) { return; }
       }
     
-      if ( pollfds[ 1 ].revents & POLLIN ) {
+      if ( sel.read( STDIN_FILENO ) ) {
 	/* input from the user needs to be fed to the network */
-	if ( !process_user_input( pollfds[ 1 ].fd ) ) {
+	if ( !process_user_input( STDIN_FILENO ) ) {
 	  if ( !network->has_remote_addr() ) {
 	    break;
 	  } else if ( !network->shutdown_in_progress() ) {
@@ -345,32 +330,32 @@ void STMClient::main( void )
 	}
       }
 
-      if ( pollfds[ 2 ].revents & POLLIN ) {
-	int signo = sigfd_read();
-
-	if ( signo == SIGWINCH ) {
-	  /* resize */
-	  if ( !process_resize() ) { return; }
-	} else if ( signo > 0 ) {
-	  /* shutdown signal */
-
-	  if ( !network->has_remote_addr() ) {
-	    break;
-	  } else if ( !network->shutdown_in_progress() ) {
-	    overlays.get_notification_engine().set_notification_string( wstring( L"Signal received, shutting down..." ), true );
-	    network->start_shutdown();
-	  }
-	}
+      if ( sel.signal( SIGWINCH ) ) {
+        /* resize */
+        if ( !process_resize() ) { return; }
       }
 
-      if ( (pollfds[ 0 ].revents)
-	   & (POLLERR | POLLHUP | POLLNVAL) ) {
+      if ( sel.signal( SIGTERM )
+           || sel.signal( SIGINT )
+           || sel.signal( SIGHUP )
+           || sel.signal( SIGPIPE )
+           || sel.signal( SIGTSTP )
+           || sel.signal( SIGCONT ) ) {
+        /* shutdown signal */
+        if ( !network->has_remote_addr() ) {
+          break;
+        } else if ( !network->shutdown_in_progress() ) {
+          overlays.get_notification_engine().set_notification_string( wstring( L"Signal received, shutting down..." ), true );
+          network->start_shutdown();
+        }
+      }
+
+      if ( sel.error( network->fd() ) ) {
 	/* network problem */
 	break;
       }
 
-      if ( (pollfds[ 1 ].revents)
-	   & (POLLERR | POLLHUP | POLLNVAL) ) {
+      if ( sel.error( STDIN_FILENO ) ) {
 	/* user problem */
 	if ( !network->has_remote_addr() ) {
 	  break;
@@ -403,10 +388,12 @@ void STMClient::main( void )
 	   && (timestamp() - network->get_latest_remote_state().timestamp > 250) ) {
 	if ( timestamp() - network->get_latest_remote_state().timestamp > 15000 ) {
 	  if ( !network->shutdown_in_progress() ) {
+	    overlays.get_notification_engine().set_notification_string( wstring( L"Timed out waiting for server..." ), true );
 	    network->start_shutdown();
 	  }
+	} else {
+	  overlays.get_notification_engine().set_notification_string( connecting_notification );
 	}
-	overlays.get_notification_engine().set_notification_string( connecting_notification );
       } else if ( (network->get_remote_state_num() != 0)
 		  && (overlays.get_notification_engine().get_notification_string()
 		      == connecting_notification) ) {
@@ -414,11 +401,16 @@ void STMClient::main( void )
       }
 
       network->tick();
+
+      const Network::NetworkException *exn = network->get_send_exception();
+      if ( exn ) {
+        overlays.get_notification_engine().set_network_exception( *exn );
+      } else {
+        overlays.get_notification_engine().clear_network_exception();
+      }
     } catch ( Network::NetworkException e ) {
       if ( !network->shutdown_in_progress() ) {
-	wchar_t tmp[ 128 ];
-	swprintf( tmp, 128, L"%s: %s", e.function.c_str(), strerror( e.the_errno ) );
-	overlays.get_notification_engine().set_notification_string( wstring( tmp ) );
+        overlays.get_notification_engine().set_network_exception( e );
       }
 
       struct timespec req;
