@@ -41,6 +41,10 @@ void ConditionalOverlayCell::apply( Framebuffer &fb, uint64_t confirmed_epoch, i
     return;
   }
 
+  if ( replacement.is_blank() && fb.get_cell( row, col )->is_blank() ) {
+    flag = false;
+  }
+
   if ( unknown ) {
     if ( flag && ( col != fb.ds.get_width() - 1 ) ) {
       fb.get_mutable_cell( row, col )->renditions.underlined = true;
@@ -49,12 +53,6 @@ void ConditionalOverlayCell::apply( Framebuffer &fb, uint64_t confirmed_epoch, i
   }
 
   if ( !(*(fb.get_cell( row, col )) == replacement) ) {
-    /*
-    if ( replacement.is_blank() && fb.get_cell( row, col )->is_blank() ) {
-      return;
-    }
-    */
-
     *(fb.get_mutable_cell( row, col )) = replacement;
     if ( flag ) {
       fb.get_mutable_cell( row, col )->renditions.underlined = true;
@@ -153,7 +151,9 @@ void ConditionalCursorMove::apply( Framebuffer &fb, uint64_t confirmed_epoch ) c
 
 NotificationEngine::NotificationEngine()
   : last_word_from_server( timestamp() ),
+    last_acked_state( timestamp() ),
     message(),
+    message_is_network_exception( false ),
     message_expiration( -1 )
 {}
 
@@ -188,15 +188,32 @@ void NotificationEngine::apply( Framebuffer &fb ) const
   /* write message */
   wchar_t tmp[ 128 ];
 
+  /* We want to prefer the "last contact" message if we simply haven't
+     heard from the server in a while, but print the "last reply" message
+     if the problem is uplink-only. */
+
+  double since_heard = (double)(now - last_word_from_server) / 1000.0;
+  double since_ack = (double)(now - last_acked_state) / 1000.0;
+  const char server_message[] = "contact";
+  const char reply_message[] = "reply";
+
+  double time_elapsed = since_heard;
+  const char *explanation = server_message;
+
+  if ( reply_late( now ) && (!server_late( now )) ) {
+    time_elapsed = since_ack;
+    explanation = reply_message;
+  }
+
   if ( message.empty() && (!time_expired) ) {
     return;
   } else if ( message.empty() && time_expired ) {
-    swprintf( tmp, 128, L"mosh: Last contact %.0f seconds ago. [To quit: Ctrl-^ .]", (double)(now - last_word_from_server) / 1000.0 );
+    swprintf( tmp, 128, L"mosh: Last %s %.0f seconds ago. [To quit: Ctrl-^ .]", explanation, time_elapsed );
   } else if ( (!message.empty()) && (!time_expired) ) {
     swprintf( tmp, 128, L"mosh: %ls [To quit: Ctrl-^ .]", message.c_str() );
   } else {
-    swprintf( tmp, 128, L"mosh: %ls (%.0f s without contact.) [To quit: Ctrl-^ .]", message.c_str(),
-	      (double)(now - last_word_from_server) / 1000.0 );
+    swprintf( tmp, 128, L"mosh: %ls (%.0f s without %s.) [To quit: Ctrl-^ .]", message.c_str(),
+	      time_elapsed, explanation );
   }
 
   wstring string_to_draw( tmp );
@@ -260,6 +277,27 @@ void NotificationEngine::adjust_message( void )
   }  
 }
 
+int NotificationEngine::wait_time( void ) const
+{
+  uint64_t next_expiry = INT_MAX;
+
+  uint64_t now = timestamp();
+
+  next_expiry = std::min( next_expiry, message_expiration - now );
+
+  if ( need_countup( now ) ) {
+    uint64_t countup_interval = 1000;
+    if ( ( now - last_word_from_server ) > 60000 ) {
+      /* If we've been disconnected for 60 seconds, save power by updating the
+         display less often.  See #243. */
+      countup_interval = Network::ACK_INTERVAL;
+    }
+    next_expiry = std::min( next_expiry, countup_interval );
+  }
+
+  return next_expiry;
+}
+
 void OverlayManager::apply( Framebuffer &fb )
 {
   predictions.cull( fb );
@@ -269,31 +307,6 @@ void OverlayManager::apply( Framebuffer &fb )
   title.apply( fb );
 }
 
-int OverlayManager::wait_time( void )
-{
-  uint64_t next_expiry = INT_MAX;
-
-  uint64_t now = timestamp();
-
-  uint64_t message_delay = notifications.get_message_expiration() - now;
-
-  if ( message_delay < next_expiry ) {
-    next_expiry = message_delay;
-  }
-
-  if ( notifications.need_countup( now ) && ( next_expiry > 1000 ) ) {
-    next_expiry = 1000;
-  }
-
-  if ( predictions.timing_tests_necessary()
-       && predictions.active()
-       && ( next_expiry > 50 ) ) {
-    next_expiry = 50;
-  }
-
-  return next_expiry;
-}
-
 void TitleEngine::set_prefix( const wstring s )
 {
   prefix = deque<wchar_t>( s.begin(), s.end() );
@@ -312,7 +325,8 @@ void PredictionEngine::apply( Framebuffer &fb ) const
 {
   bool show = (display_preference != Never) && ( srtt_trigger
 						 || glitch_trigger
-						 || (display_preference == Always) );
+						 || (display_preference == Always)
+						 || (display_preference == Experimental) );
 
   if ( show ) {
     for ( cursors_type::const_iterator it = cursors.begin();
@@ -448,7 +462,11 @@ void PredictionEngine::cull( const Framebuffer &fb )
 		   );
 	  */
 
-	  kill_epoch( j->tentative_until_epoch, fb );
+	  if ( display_preference == Experimental ) {
+	    j->reset();
+	  } else {
+	    kill_epoch( j->tentative_until_epoch, fb );
+	  }
 	  /*
 	  if ( j->display_time != uint64_t(-1) ) {
 	    fprintf( stderr, "TIMING %ld - %ld (TENT)\n", time(NULL), now - j->display_time );
@@ -468,8 +486,12 @@ void PredictionEngine::cull( const Framebuffer &fb )
 	  }
 	  */
 
-	  reset();
-	  return;
+	  if ( display_preference == Experimental ) {
+	    j->reset();
+	  } else {
+	    reset();
+	    return;
+	  }
 	}
 	break;
       case Correct:
@@ -535,8 +557,12 @@ void PredictionEngine::cull( const Framebuffer &fb )
 	       cursor().expiration_time,
 	       now );
       */
-      reset();
-      return;
+      if ( display_preference == Experimental ) {
+	cursors.clear();
+      } else {
+	reset();
+	return;
+      }
     }
   }
 
@@ -577,6 +603,8 @@ void PredictionEngine::new_user_byte( char the_byte, const Framebuffer &fb )
 {
   if ( display_preference == Never ) {
     return;
+  } else if ( display_preference == Experimental ) {
+    prediction_epoch = confirmed_epoch;
   }
 
   cull( fb );
@@ -797,7 +825,9 @@ void PredictionEngine::newline_carriage_return( const Framebuffer &fb )
 
 void PredictionEngine::become_tentative( void )
 {
-  prediction_epoch++;
+  if ( display_preference != Experimental ) {
+    prediction_epoch++;
+  }
 
   /*
   fprintf( stderr, "Now tentative in epoch %lu (confirmed=%lu)\n",
